@@ -1,7 +1,7 @@
 /*
  * libeio implementation
  *
- * Copyright (c) 2007,2008,2009,2010,2011,2012,2013,2016,2017,2018 Marc Alexander Lehmann <libeio@schmorp.de>
+ * Copyright (c) 2007,2008,2009,2010,2011,2012,2013,2016,2017,2018,2019 Marc Alexander Lehmann <libeio@schmorp.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modifica-
@@ -1641,7 +1641,7 @@ eio__scandir (eio_req *req, etp_worker *self)
 
 #if !HAVE_AT
 
-/* a bit like realpath, but usually faster because it doesn'T have to return */
+/* a bit like realpath, but usually faster because it doesn't have to return */
 /* an absolute or canonical path */
 static const char *
 wd_expand (struct etp_tmpbuf *tmpbuf, eio_wd wd, const char *path)
@@ -1681,6 +1681,20 @@ eio__wd_open_sync (struct etp_tmpbuf *tmpbuf, eio_wd wd, const char *path)
 #if HAVE_AT
   fd = openat (WD2FD (wd), path, O_CLOEXEC | O_SEARCH | O_DIRECTORY | O_NONBLOCK);
 
+  /* 0 is a valid fd, but we use it for EIO_CWD, so in the very unlikely */
+  /* case of fd 0 being available (almost certainly an a pplication bug) */
+  /* make sure we use another fd value */
+  #if EIO_CWD
+    error EIO_CWD must be 0
+  #endif
+  if (ecb_expect_false (fd == 0))
+    {
+      int fd2 = fcntl (fd, F_DUPFD_CLOEXEC ? F_DUPFD_CLOEXEC : F_DUPFD);
+      fcntl (fd2, F_SETFD, FD_CLOEXEC);
+      eio__close (fd);
+      fd = fd2;
+    }
+
   if (fd < 0)
     return EIO_INVALID_WD;
 #endif
@@ -1926,7 +1940,7 @@ eio_execute (etp_worker *self, eio_req *req)
       case EIO_RMDIR:     /* complications arise because "." cannot be removed, so we might have to expand */
                           req->result = req->wd && SINGLEDOT (req->ptr1)
                              ? rmdir (req->wd->str)
-                             : unlinkat  (dirfd, req->ptr1, AT_REMOVEDIR); break;
+                             : unlinkat (dirfd, req->ptr1, AT_REMOVEDIR); break;
       case EIO_MKDIR:     req->result = mkdirat   (dirfd, req->ptr1, (mode_t)req->int2); break;
       case EIO_RENAME:    req->result = eio__renameat2 (
                             dirfd,
